@@ -126,7 +126,6 @@ do_open(char *tool, char *name, void **domain, bool bytag, int timeout, char *us
 	ni_status status;
 	struct sockaddr_in server;
 	ni_id rootdir;
-	void *localni;
 
 	if (bytag)
 	{
@@ -153,26 +152,7 @@ do_open(char *tool, char *name, void **domain, bool bytag, int timeout, char *us
 	}
 	else
 	{
-		if (!strcmp(name, "."))
-		{
-			status = ni_open(NULL, ".", domain);
-		}
-		else
-		{
-			status = ni_open(NULL, ".", &localni);
-			if (status != NI_OK)
-			{
-				fprintf(stderr, "%s: can't connect to server for local domain\n", tool);
-				return status;
-			}
-			ni_setabort(localni, 1);
-			ni_setreadtimeout(localni, timeout);
-			ni_setwritetimeout(localni, timeout);
-		
-			status = ni_open(localni, name, domain);
-			ni_free(localni);
-		}
-
+		status = ni_open(NULL, name, domain);
 		if (status != NI_OK)
 		{
 			fprintf(stderr, "%s: can't connect to server for domain %s\n", tool, name);
