@@ -43,6 +43,8 @@ extern id configManager;
 extern id statistics;
 
 extern BOOL shutting_down;
+extern BOOL debug_enabled;
 extern BOOL statistics_enabled;
+extern BOOL coredump_enabled;
 
 #define DefaultName "lookup daemon v2"
